//XmlMesh.cpp
#include "XmlMesh.h"
#include "../xmlnode/XmlNode.h"
#include "../xmlnode/StringUtil.h"
#include "EDefine.h"
#include "EStaticMesh.h"

extern LPDIRECT3DDEVICE9   g_d3d_Device; 

XmlMesh::XmlMesh()
{
}
//-----------------------------------------------------------------------------
XmlMesh::~XmlMesh()
{
	m_vertex.clear();
	m_index.clear();
}
//-----------------------------------------------------------------------------
const char*	XmlMesh::GetClassName()
{
	return "XmlMesh";
}
//-----------------------------------------------------------------------------
void XmlMesh::ConvertMeshInfo()
{
	XmlNode* pVertexXmlNode = FindFirstChild( "Vertex" );
	if( pVertexXmlNode == NULL )
		return;

	XmlMesh* pXmlMesh = (XmlMesh*)pVertexXmlNode;
	pXmlMesh->FetchVertex( m_vertex );
	
	XmlNode* pFaceXmlNode = FindFirstChild( "Face" );
	if( pFaceXmlNode  == NULL )
		return;
	
	pXmlMesh = (XmlMesh*)pFaceXmlNode;
	pXmlMesh->FetchFace( m_index );
}
//-----------------------------------------------------------------------------
void XmlMesh::OnUpdate()
{
	if( *GetName() == "mesh" )
	{
		ConvertMeshInfo(); 	
		DetachChild(); //ConvertMeshInfo() ķδ Vertex Index  Ѵ.
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchVertex( std::vector<VertexInfo>& vertexArr )
{
	//"Vertex" Ʈ - "Number" Ʈ ϱ
	XmlNode* pXmlNode = FindFirstChild( "Number" );
	if( pXmlNode == NULL )
		return;

	int number = ToInt( *pXmlNode->GetData() );
	if( number < 3 )
		return;

	//"Vertex" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for( int index = 0; iter != endIter; ++iter, ++index )
	{
		XmlNode* xmlNode = *iter;

		VertexInfo info;
		info.index = index;
		info.p = ToVector3( *xmlNode->GetData() );		
		vertexArr.push_back( info );
	}
}
//-----------------------------------------------------------------------------
void XmlMesh::FetchFace( std::vector<base::Index3>& indexArr )
{
	//"Face" Ʈ - "Number" Ʈ ϱ
	XmlNode* pXmlNode = FindFirstChild( "Number" );
	if( pXmlNode == NULL )
		return;

	int number = ToInt( *pXmlNode->GetData() );
	if( number < 1 )
		return;

	//"Face" Ʈ "data" Ʈ ϱ
	XmlNodeVector nodeArr;
	FindChild( "data", nodeArr );
	if( nodeArr.size() != number )
		return;

	XmlNodeVector::iterator iter = nodeArr.begin();
	XmlNodeVector::iterator endIter = nodeArr.end();
	for(; iter != endIter; ++iter )
	{
		XmlNode* xmlNode = *iter;
		base::Index3 index = ToIndex3( *xmlNode->GetData() );
		indexArr.push_back( index );
	}
}
//-----------------------------------------------------------------------------
ENode* XmlMesh::OnXmlToENode( ENode* pParent )
{
	ENode* pNode = NULL;
	if( m_vertex.size() > 0 )
	{
		EStaticMesh* pStaticMesh = new EStaticMesh( g_d3d_Device );
		pStaticMesh->CreateVertexBuffer( this );
		pNode = pStaticMesh;
	}
	else
	{
		pNode = new ENode( g_d3d_Device );
	}

	return pNode;
}
//-----------------------------------------------------------------------------
void XmlMesh::CreateMesh( ENode*& pParent )
{
	ENode* pNode;
	if( pParent == NULL )
	{
		pNode = pParent = new ENode( g_d3d_Device );
	}
	else
	{
		pNode = OnXmlToENode( pParent );
		pParent->AttachChild( pNode );
	}

	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlMesh* pChild = (XmlMesh*)*iter;
		pChild->CreateMesh( pNode );
	}	
}
