//XmlXmlNode.cpp
#include <assert.h>
#include <iostream>
#include <stdio.h>
#include "XmlNode.h"


void _split(const std::string line, const std::string separator, std::vector<std::string>& tokens) 
{
    size_t seplen = separator.length();
    size_t start = 0;
    size_t end = line.length();
 
    while ((start >= 0) && (start < end)) 
	{
        start = start + seplen;
        size_t stop = line.find(separator, start);
        if ((stop < 0) || (stop > end)) 
			stop = end;
        tokens.push_back(start==seplen ? line.substr(0, stop)  : line.substr(start, stop-start));
        start = stop;
    }
}
//-----------------------------------------------------------------------------
XmlNode::XmlNode()
: m_pParent( NULL ) 
, m_child( NULL )
{	
}
//-----------------------------------------------------------------------------
XmlNode::~XmlNode()
{	
	//Ӽ 
	{
		AttributeList::iterator iter = m_attribute.begin();
		AttributeList::iterator endIter = m_attribute.end();
		for( ; iter != endIter; ++iter )
		{
			delete (*iter);
		}
		m_attribute.clear();
	}
	//ڽ 
	DetachChild();
}
//-----------------------------------------------------------------------------
const char* XmlNode::GetClassName()
{
	return "XmlNode";
}
//-----------------------------------------------------------------------------
XmlNode* XmlNode::AttachChild( XmlNode* pChild )
{
	pChild->m_pParent = this;
	m_child.push_back( pChild );
	return pChild;
}
//-----------------------------------------------------------------------------
void XmlNode::DetachChild()
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		delete (*iter);
	}
	m_child.clear();
}
//-----------------------------------------------------------------------------
void XmlNode::DetachChild( XmlNode* pChild )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		if( pChild == *iter )
		{
			delete (*iter);
			m_child.erase( iter );
			break;
		}
	}
}
//-----------------------------------------------------------------------------
XmlNode* XmlNode::GetParent()
{
	return m_pParent;
}
//-----------------------------------------------------------------------------
XmlNode* XmlNode::GetRoot()
{
	return m_pParent ? m_pParent->GetRoot() : this;
}
//---------------------------------------------------------------------------
XmlNode& XmlNode::SetData( std::string data )
{
	m_data = data;
	return *this;
}
//-----------------------------------------------------------------------------
XmlNode& XmlNode::SetData( const char* pData )
{
	m_data = pData;
	return *this;
}
//-----------------------------------------------------------------------------
std::string*  XmlNode::GetData()
{
	return &m_data;
}
//-----------------------------------------------------------------------------
std::string* XmlNode::GetName()
{
	return &m_name;
}
//-----------------------------------------------------------------------------
XmlNode& XmlNode::SetName( const char* szName )
{
	m_name = szName;
	return *this;
}
//-----------------------------------------------------------------------------
XmlNode& XmlNode::AddAttribute( const char* pName, const char* pValue )
{
	m_attribute.push_back(  new Attribute( pName, pValue ) );
	return *this;
}
//-----------------------------------------------------------------------------
void XmlNode::FindNodeAll( const std::vector<std::string>& tok, int iDepth, XmlNodeVector& nodeArr )
{
	const char* pName = tok[iDepth].c_str();
	if( *this->GetName() != pName )
		return;

	if( tok.size() == unsigned(iDepth + 1) )
	{
		nodeArr.push_back( this );
		return;
	}

	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		XmlNode* pChild = *iter;
		pChild->FindNodeAll( tok, iDepth + 1, nodeArr );
	}
}
//-----------------------------------------------------------------------------
XmlNode* XmlNode::FindFirstChild( const char* pName )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		XmlNode* pChild = *iter;
		if( *pChild->GetName() == pName )
			return pChild;
	}

	return NULL;
}
//-----------------------------------------------------------------------------
void XmlNode::FindChild( const char* pName, XmlNodeVector& nodeArr )
{
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();
	for( ; iter != endIter; ++iter )
	{
		XmlNode* pChild = *iter;
		if( *pChild->GetName() == pName )
			nodeArr.push_back( pChild );
	}
}
//-----------------------------------------------------------------------------
void XmlNode::FindNode( const char* pName, XmlNodeVector& nodeArr )
{
	std::vector<std::string> tok;
	_split( pName, "/", tok );

	FindNodeAll( tok, 0, nodeArr );
}
//-----------------------------------------------------------------------------
XmlNode* XmlNode::FindNode( const char* pName, const char* attrName, const char* pAttrValue )
{
	if( *this->GetName() == pName && IsExsitAttribute( attrName, pAttrValue ) )
			return this;

	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	XmlNode* pXmlNode = NULL;

	for( ; iter != endIter; ++iter )
	{
		XmlNode* pChild = *iter;
		pXmlNode = pChild->FindNode( pName, attrName, pAttrValue );
		if( pXmlNode )
			return pXmlNode;
	}

	return NULL;
}
//-----------------------------------------------------------------------------
void XmlNode::Traverse()
{
	OnUpdate();
	ChildList::iterator iter = m_child.begin();
	ChildList::iterator endIter = m_child.end();

	for( ; iter != endIter; ++iter )
	{
		XmlNode* pChild = *iter;
		pChild->Traverse();
	}	
}
//-----------------------------------------------------------------------------
void XmlNode::OnUpdate()
{
}
//-----------------------------------------------------------------------------
bool XmlNode::IsExsitAttribute( const char* attrName, const char* pAttrValue )
{
	AttributeList::iterator iter = m_attribute.begin();
	AttributeList::iterator endIter = m_attribute.end();
	for( ; iter != endIter; ++iter )
	{
		Attribute* pAttr = *iter;
		if( pAttr->name == attrName && pAttr->value == pAttrValue )
			return true;
	}

	return false;
}
//-----------------------------------------------------------------------------
void XmlNode::PrintAll( XmlNode* pXmlNode )
{
	std::cout << "XmlNode Name : " << pXmlNode->GetParent() << " : " << pXmlNode << " : " 
		<< pXmlNode->GetName()->c_str() << std::endl;
	if( pXmlNode->m_child.empty() == true )
		return;
	
	ChildList::iterator iter = pXmlNode->m_child.begin();
	ChildList::iterator endIter = pXmlNode->m_child.end();
	for( ; iter != endIter; ++iter )
	{
		PrintAll( *iter );
	}
}
//-----------------------------------------------------------------------------
void XmlNode::WriteXml( XmlNode* pNode, TiXmlNode* pXml )
{
	TiXmlElement* xmlNode = (TiXmlElement*)pXml;
	if( pNode->GetName() && pNode->GetName()->empty() == false )
	{
		xmlNode = new TiXmlElement( pNode->GetName()->c_str() );  
		pXml->LinkEndChild( xmlNode ); 

		if(pNode->GetData() && pNode->GetData()->empty() == false )
		{
			xmlNode->LinkEndChild( new TiXmlText( pNode->GetData()->c_str() )); 
		}

		AttributeList::iterator iter = pNode->m_attribute.begin();
		AttributeList::iterator endIter = pNode->m_attribute.end();
		for( ; iter != endIter; ++iter )
		{
			Attribute* pAttr = *iter;
			xmlNode->SetAttribute( pAttr->name.c_str(), pAttr->value.c_str() );
		}
	}

	ChildList::iterator iter = pNode->m_child.begin();
	ChildList::iterator endIter = pNode->m_child.end();
	for( ; iter != endIter; ++iter )
	{
		WriteXml( *iter, (TiXmlNode*)xmlNode );
	}
}
//-----------------------------------------------------------------------------
//_ReadXml  : 
// XmlNode* dataXmlNode			 ȣ⿡ θ XmlNode ش.
// TiXmlNode* xmlNode			 ȣ⿡ TiXmlXmlNode ش.
// XmlNode*& outXmlNode			 ȣ⿡ Ʈ XmlNode Ѵ.
void _ReadXml( XmlNode* dataNode, TiXmlNode* xmlNode, XmlNode*& outNode )
{
	if(xmlNode)
	{
		switch(xmlNode->Type())
	   	{
		case TiXmlNode::COMMENT:
		    //std::cout << "COMMENT: " << xmlNode->Value() << std::endl;
		    break;
	   	case TiXmlNode::DECLARATION:
		    //std::cout << "DECLARATION: " << std::endl;
	   	    break;

	   	case TiXmlNode::ELEMENT:
	   		{
				//std::cout << "ELEMENT: [" << xmlNode->ToElement()->Value() << "]" << std::endl;
				if( outNode == NULL )
				{
					dataNode = outNode = new XmlNode;
				}
				else
				{
					XmlNode* child = new XmlNode;
					dataNode->AttachChild( child );
					dataNode = child;
				}

				dataNode->SetName( xmlNode->ToElement()->Value() );
	   			TiXmlAttribute *pAttr = xmlNode->ToElement()->FirstAttribute();
	   			while(pAttr)
	   			{
					dataNode->AddAttribute( pAttr->Name(), pAttr->Value() );
					//std::cout << pAttr->Name() << "=\"" << pAttr->Value() << "\"" << std::endl;
	   	   			pAttr = pAttr->Next();
	   			}
			}
	   		break;

	   	case TiXmlNode::TEXT:
		    TiXmlText* pText = xmlNode->ToText();
		    if(pText)
		    {
			    dataNode->SetData( pText->Value() );
			    //std::cout << "TEXT:  " << pText->Value() << std::endl;
		    }
		    break;
	   	}

	   	for(TiXmlNode *pXmlNext = xmlNode->FirstChild(); pXmlNext; pXmlNext = pXmlNext->NextSibling()) 
		    _ReadXml( dataNode, pXmlNext, outNode );
	}
}
//-----------------------------------------------------------------------------
void WriteXml( const char* szName, XmlNode* root )
{
	TiXmlDocument doc;  
	TiXmlDeclaration* decl = new TiXmlDeclaration( "1.0", "euc-kr", "" );  
	doc.LinkEndChild( decl ); 

	XmlNode::WriteXml( root, &doc );
	doc.SaveFile( szName );
}
//-----------------------------------------------------------------------------
XmlNode* ReadXml( const char* szName )
{
	TiXmlDocument doc;
	if ( doc.LoadFile( szName ) == false )
	{
		return NULL;
	} 

	XmlNode* parent = NULL;
	_ReadXml( NULL, &doc, parent );
	return parent;
}
//-----------------------------------------------------------------------------
XmlNode* CreateXmlNode( char* szName )
{
	XmlNode* pXmlNode = new XmlNode;
	pXmlNode->SetName( szName );
	return pXmlNode;
}
